<?php

namespace blocks\query;

use types\FilterType;
use types\FilterTermType;

class FiltersManager
{
    private ?string $current_block_id;

    function __construct(
    ) {
        $this->current_block_id = get_current_element_ID();

        require_once get_template_directory() . "/types/FilterTermType.php";
        require_once get_template_directory() . "/types/FilterType.php";
    }

    /**
     * Returns an array of the different filters
     * @return FilterType[]|null
     */
    private function filters(mixed $current_block): ?array
    {
        $all_filters = array();

        foreach ($current_block["filters"] as $filter) {
            if ($filter["type"] == "post-type") {
                $type = "pt";
                $by = "Content";
                $multi = "false";
                $terms = array();

                foreach ($current_block["args"]["post_type"] as $post_type) {
                    $terms[] = array(
                        "key" => null,
                        "value" => $post_type,
                        "name" => get_post_type_labels(get_post_type_object($post_type))->singular_name
                    );
                }
            } elseif ($filter["type"] == "taxonomy") {
                $type = "tx";
                $by = get_taxonomy_labels(get_taxonomy($filter["taxonomy"]))->singular_name;
                $multi = ($filter["multi"] == 1) ? "true" : "false";
                $terms = array();

                $filter_tax_terms = get_terms(array(
                    "taxonomy" => $filter["taxonomy"],
                    "hide_empty" => true
                ));

                // iterate over the options of each filter
                foreach ($filter_tax_terms as $term) {
                    $terms[] = new FilterTermType(
                        key: $filter['taxonomy'],
                        value: $term->term_id,
                        name: $term->name
                    );
                }
            } else {
                return null;
            }

            $all_filters[] = new FilterType(
                type: $type,
                by: $by,
                multi: $multi,
                terms: $terms
            );
        }

        return $all_filters;
    }

    /**
     * Displays the filter UI using FilterType array
     * @param string $title title of the filter section
     * @param ?FilterType[] $filters list of the FilterType used for filtering
     * @return void
     */
    public function display(string $title, ?array $filters): void
    {
        echo '
                <div id="' . $this->current_block_id . '-filter" class="query-filter-wrap">
                    <div class="query-filter-items">
    
                        <h4 class="query-filter-menu-head mb-3 mb-4 d-flex justify-content-between align-items-end">' .
                            $title . ' <span class="query-filter-clear">Clear All</span>
                        </h4>';

        if ($filters != null) {
            foreach ($filters as $filter) {
                if (get_taxonomy($filter->by)) {
                    $taxonomy_name = get_taxonomy_labels(get_taxonomy($filter->by))->name;
                } else {
                    $taxonomy_name = $filter->by;
                }

                echo '
                        <div class="query-filter">
                            <h6 class="query-filter-head mb-2">By ' . $taxonomy_name . '</h6>
                            <ul class="query-filter-list" data-multi="' . $filter->multi . '">';

                foreach ($filter->terms as $term) {
                    $terms_key = $term->key;
                    $data_filter_key = (($terms_key != null) ? 'data-filter-key="' . $terms_key . '"' : null);

                    echo '
                                <li class="query-filter-item d-flex align-items-center"' . '
                                    data-filter-type="' . $filter->type . '"' .
                    $data_filter_key . ' 
                                    data-filter-value="' . $term->value . '">
                                    
                                    <i class="icon far fa-square fa-sm fa-fw mr-2"></i>
                                    <span class="label">' . $term->name . '</span>
                                </li>';
                }
                echo '
                            </ul>
                        </div>';
            }
            echo '
                    </div>
                </div>';
        }
    }


    /**
     * Displays the filter UI using $query_block[$current_block_ID]
     * @param mixed $current_block
     * @return void
     */
    public function display_legacy(mixed $current_block): void {
        $filters = self::filters($current_block);

        $this::display(
            title: $current_block['display']['filter']['head'],
            filters: $filters
        );
    }
}
